#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"

#define SCREEN_WIDTH    480
#define SCREEN_HEIGHT   480

#define NB_CELLS        30
#define CELL_SIZE       (SCREEN_HEIGHT / NB_CELLS)

#define RED_RATIO   0.5     // % of red in the initial state
#define EMPTY       0.1     // % of empty cells
#define LIKENESS    0.3     // % of wanted neighbors of the same color

int grid[NB_CELLS][NB_CELLS];

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Segregation 2", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));
    srand(time(NULL));

    // initialize the palette
    Color   palette[3];

    palette[0] = Color( 70,  70,  70); // empty cell
    palette[1] = Color(220,   0,   0); // red
    palette[2] = Color(  0, 150,   0); // green

    // initialize the grid
    for (int y = 0; y < NB_CELLS; y++)
        for (int x = 0; x < NB_CELLS; x++)
            grid[x][y] = 0;

    int nbOccupied = NB_CELLS * NB_CELLS * (1 - EMPTY); // number of occupied cells
    int nbRed = nbOccupied * RED_RATIO; // number of red cells

    for (int i = 0; i < nbOccupied; i++)
    {
        // find an empty cell
        while (true)
        {
            int x = rand() % NB_CELLS;
            int y = rand() % NB_CELLS;

            if (grid[x][y] == 0)
            {
                // fill the cell
                if (i < nbRed)
                    grid[x][y] = 1;
                else
                    grid[x][y] = 2;

                break;
            }
        }
    }

    // main loop
    while (sys.isQuitRequested() == false)
    {
        // draw the grid
        for (int y = 0; y < NB_CELLS; y++)
            for (int x = 0; x < NB_CELLS; x++)
            {
                gfx.rectFill(      x * CELL_SIZE,           y * CELL_SIZE,
                             (x + 1) * CELL_SIZE - 2, (y + 1) * CELL_SIZE - 2,
                             palette[grid[x][y]]);
            }

        gfx.render();
        sys.wait(1);
        sys.processEvents();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
